/*
This is the actual Focusing procs and variables.

var/IconFocus[0]	- This is the list of some icons. You add these
					at run time, or in the code. It is basically a
					list in the format list("Type" = Value). This can
					be accessed like IconFocus["Nose" = 16]. This
					particular way is not the greatest, because it only
					shows one value, which, unless both x and y pixel
					coordinates are the same is not accurate. I would
					probably recommend IconFocus["Nosex" = 16] and
					IconFocus["Nosey" = 10].

datum/var/Focus[0]	- This is basically the same as IconFocus[0], but
					it can be used to assign different values to
					different types. Let's use hands, /mob/Wizard's
					hand may be at 8,11, but /mob/Goblin's hand may be
					at 5,13. This is the main purpose of this function,
					to allow different types to have overlays and other
					graphical features in different places with only
					one icon. If you wanted to access either of the two
					previous type path's hands, you would simply put
					datum.Focus["Handx"] or datum.Foucs["Handy"] to
					access any datum's hand, once the correct variables
					are assigned.

datum/var/Images[0]	- This is a list of all the images to be put on any
					movable atom when it moves. Whenever it moves, it
					overrides the current overlays with a new list,
					then puts on all of the images. This can also be
					handy even if you don't use the Focus() proc. This
					variable allows you to add and remove overlays just
					as easily or more easily than the normal way. The
					reason this primarily updates when the atom moves,
					is this also allows for directional specific
					images. Look below on more directional specific
					information.

datum/var/IsDirSpecific	- This is the variable that defines if a datum
					will change when it's in another datum's Image[0]
					list. It is FALSE, or 0 by default. The changing
					of an icon's direction while in a players overlays
					can be essential in many cases. For example, if a
					player has a four direction icon, and they have a
					sword, if they run east, or north, or south, or
					west, their sword probably would need to change.

datum/proc/InitializeFocus()	- This is called during datum/New()
					and by default does nothing. It is used to add
					values or other lists to the default Focus[0]
					variable. In this way, you do not overrite any
					other of the initial values, but more can still be
					added. When using this proc, you should always
					call a parent, either by stating ..() somewhere
					in the procedure, setting the value of . to ..(),
					or at the end returning ..().
						EX:
							InitializeFocus()
								..()
								<Code>
							InitializeFocus()
								. = ..()
								<Code>
							InitializeFocus()
								<Code>
								return ..()

datum/proc/FocusedImages(Dir = NORTH)	- This is the procedure called
					by a movable atom moving. It can be called at any
					time, but by default is only called when a movable
					atom moves. It resets the overlays to a new list,
					then goes through the images and adds new ones with
					the direction that is specified, by default, NORTH.

datum/proc/IconImageShift(Dir = NORTH , icon/I)	- This is called by
					datum.FocusedImages(), because the
					datum.FocusedImages() procedure removes all
					overlays, then makes a new icon, or datum to be
					added. If the type to be added is an icon, this is
					called, if the type is a datum, it calls
					datum.DatumImageShift() instead. This procedure
					does nothing by default, but it can be used to
					change the pixel locations when the object is in a
					different direction. For example, if a player has
					a sword icon in their Focus[0] list, and they turn
					to a different direction, a new sword icon will be
					made, and this procedure will be called. This would
					be useful when the sword is at different heights
					when they are facing different directions.

datum/proc/DatumImageShift(Dir = NORTH , icon/I , datum/D)	- This is
					called by datum.FocusedImages(), because the
					datum.FocusedImages() procedure removes all
					overlays, then makes a new icon, or datum to be
					added. If the type to be added is a datum, this is
					called, if the type is an icon, it calls
					datum.IconImageShift() instead. This procedure
					does nothing by default, but it can be used to
					change the pixel locations when the object is in a
					different direction. For example, if a player has
					a sword object in their Focus[0] list, and they
					turn to a different direction, a new sword pbject
					will be made, and this procedure will be called.
					This would be useful when the sword is at different
					heights when they are facing different directions.

proc/Focus(icon/I , A = 1 , B = 1 , C = 1 , D = 1)
					- This is the function that makes the icon be
					placed correctly. The first argument is the icon to
					be focused. The second is that icon's starting x,
					the third is the icon's starting y, fourth is the
					ending x, and fifth is the ending y. See the demo
					on how to call this correctly. It will return 1 on
					completion, and 0 on failure.

*/

var/IconFocus[0]

datum

	New() { InitializeFocus() ; return ..() }

	var

		Focus[0]
		Images[0]
		IsDirSpecific = FALSE

	proc

		InitializeFocus()
			return ..()

		FocusedImages(Dir = NORTH)
			src:overlays = new/list
			for(var/datum/D in Images)
				if(isicon(D))
					var/icon/I = new(D)
					IconImageShift(Dir,I) ; src:overlays += I
				else
					if(D.IsDirSpecific) D:dir = Dir
					var/icon/I = new(D:icon,D:icon_state,D:dir)
					DatumImageShift(Dir,I,D) ; src:overlays += I

		IconImageShift(Dir = NORTH , icon/I)
			return ..()

		DatumImageShift(Dir = NORTH , icon/I , datum/D)
			return ..()

proc

	Focus(icon/I , A = 1 , B = 1 , C = 1 , D = 1)
		if(!I) return 0
		var/E = 1 , x1 = A , y1 = B , x2 = C , y2 = D
		while(E)
			E = 0
			if(x1 < x2) {E = 1 ; I.Shift(EAST,1,1) ; x1++}
			if(y1 < y2) {E = 1 ; I.Shift(NORTH,1,1) ; y1++}
			if(x1 > x2) {E = 1 ; I.Shift(WEST,1,1) ; x1--}
			if(y1 > y2) {E = 1 ; I.Shift(SOUTH,1,1) ; y1--}
		return 1

atom/movable/Move(NewLoc,Dir) { FocusedImages(Dir) ; return ..() }
